# -*- encoding: binary -*-
# internal implementation details here, do not rely on this in your code
require "socket"

module MogileFS::SocketCommon
  attr_reader :mogilefs_addr

  def post_init(host, port)
    @mogilefs_addr = "#{host}:#{port}"
    Socket.const_defined?(:TCP_NODELAY) and
      setsockopt(Socket::IPPROTO_TCP, Socket::TCP_NODELAY, 1)
    self
  end

  def unreadable_socket!
    raise MogileFS::UnreadableSocketError,
          "#@mogilefs_addr never became readable"
  end

  def request_truncated!(written, expect)
    raise MogileFS::RequestTruncatedError,
          "request truncated (sent #{written} expected #{expect})"
  end

  SEP_RE = /\A(.*?#{Regexp.escape("\n")})/
  def timed_gets(timeout = 5)
    unless defined?(@rbuf) && @rbuf
      @rbuf = timed_read(1024, "", timeout) or return # EOF
    end
    begin
      @rbuf.sub!(SEP_RE, "") and return $1
      tmp ||= ""
      if timed_read(1024, tmp, timeout)
        @rbuf << tmp
      else
        # EOF, return the last buffered bit even without SEP_RE matching
        # (not ideal for MogileFS, this is an error)
        return @rbuf.empty? ? nil : @rbuf.slice!(0, @rbuf.size)
      end
    end while true
  end

  def read(size, buf = "", timeout = 5)
    timed_read(size, buf, timeout) or return # nil/EOF

    while size > buf.bytesize
      tmp ||= ""
      timed_read(size - buf.bytesize, tmp, timeout) or return buf # truncated
      buf << tmp
    end

    buf # full read
  end

  def readpartial(size, buf = "", timeout = 5)
    timed_read(size, buf, timeout) or raise EOFError, "end of file reached"
  end
end
