# -*- encoding: binary -*-
require "./test/fresh"

class TestMogstoredRack < Test::Unit::TestCase
  include TestFreshSetup
  def setup
    setup_mogilefs
  end

  def test_md5_check
    add_host_device_domain
    client = MogileFS::MogileFS.new :hosts => @hosts, :domain => @domain
    node = "#@test_host:#@mogstored_http_port"
    pid = fork do
      # not modifying this hash in the same process
      MogileFS::HTTPFile::MD5_TRAILER_NODES[node] = true
      client.store_content("md5_me", nil, "HELLO WORLD")
    end
    _, status = Process.waitpid2(pid)
    assert status.success?, status.inspect
    assert_equal "HELLO WORLD", client.get_file_data("md5_me")
  end

  def setup_mogstored
    @docroot = Dir.mktmpdir(["mogfresh", "docroot"])
    @mogstored_mgmt = TCPServer.new(@test_host, 0)
    @mogstored_http = TCPServer.new(@test_host, 0)
    @mogstored_mgmt_port = @mogstored_mgmt.addr[1]
    @mogstored_http_port = @mogstored_http.addr[1]
    @mogstored_conf = Tempfile.new(["mogstored", "conf"])
    @mogstored_pid = Tempfile.new(["mogstored", "pid"])
    @mogstored_conf.write <<EOF
pidfile = #{@mogstored_pid.path}
maxconns = 1000
mgmtlisten = #@test_host:#{@mogstored_mgmt_port}
server = none
docroot = #@docroot
EOF
    @mogstored_conf.flush
    @mogstored_mgmt.close

    unicorn_setup

    x!("mogstored", "--daemon", "--config=#{@mogstored_conf.path}")
    wait_for_port @mogstored_mgmt_port
  end

  # I would use Rainbows! + *Threads + Ruby 1.9.3 in production
  def unicorn_setup
    examples_dir = Dir.pwd + "/examples"
    assert File.directory?(examples_dir)
    @ru = Tempfile.new(%w(mogstored_rack .ru))
    @ru.write <<EOF
require "mogstored_rack"
use Rack::Head
run MogstoredRack.new("#@docroot")
EOF
    @ru.flush

    @unicorn_pid = Tempfile.new(%w(unicorn .pid))
    @unicorn_conf = Tempfile.new(%w(unicorn.conf .rb))
    @unicorn_stderr = Tempfile.new(%w(unicorn .stderr))
    @unicorn_stdout = Tempfile.new(%w(unicorn .stdout))
    @unicorn_conf.write <<EOF
listen "#@test_host:#{@mogstored_http_port}"
pid "#{@unicorn_pid.path}"
stderr_path "#{@unicorn_stderr.path}"
stdout_path "#{@unicorn_stdout.path}"
rewindable_input false
EOF
    @unicorn_conf.flush

    @mogstored_http.close
    x!("unicorn", "-I", examples_dir, "-E", "deployment",
       "--daemon", "--config", @unicorn_conf.path, @ru.path)
    wait_for_port @mogstored_http_port
    40.times do
      break if File.size(@unicorn_pid.path) > 0
      sleep 0.1
    end
  end

  def teardown
    pid = File.read(@unicorn_pid.path).to_i
    Process.kill(:QUIT, pid) if pid > 0
    teardown_mogilefs
    puts(@unicorn_stderr.read) if $DEBUG
  end
end if `which unicorn`.chomp.size > 0
